@@ -46,7 +46,7 @@ static const char* armPartId2name(uint32_t partId)
     case 0xa20: return "ARM1020";
     case 0xa22: return "ARM1022";
     case 0xa26: return "ARM1026";
-    case 0xb02: return "ARM11 MPCore";
+    case 0xb02: return "ARM11-MPCore";
     case 0xb36: return "ARM1136";
     case 0xb56: return "ARM1156";
     case 0xb76: return "ARM1176";
@@ -82,10 +82,12 @@ static const char* armPartId2name(uint32_t partId)
     case 0xd0d: return "Cortex-A77";
     case 0xd0e: return "Cortex-A76AE";
     case 0xd13: return "Cortex-R52";
+    case 0xd14: return "Cortex-R82AE";
     case 0xd15: return "Cortex-R82";
     case 0xd16: return "Cortex-R52+";
     case 0xd20: return "Cortex-M23";
     case 0xd21: return "Cortex-M33";
+    case 0xd24: return "Cortex-M52";
     case 0xd22: return "Cortex-M55";
     case 0xd23: return "Cortex-M85";
     case 0xd40: return "Neoverse-V1";
@@ -106,9 +108,18 @@ static const char* armPartId2name(uint32_t partId)
     case 0xd80: return "Cortex-A520";
     case 0xd81: return "Cortex-A720";
     case 0xd82: return "Cortex-X4";
+    case 0xd83: return "Neoverse-V3AE";
     case 0xd84: return "Neoverse-V3";
     case 0xd85: return "Cortex-X925";
     case 0xd87: return "Cortex-A725";
+    case 0xd88: return "Cortex-A520AE";
+    case 0xd89: return "Cortex-A720AE";
+    case 0xd8a: return "C1-Nano";
+    case 0xd8b: return "C1-Pro";
+    case 0xd8c: return "C1-Ultra";
+    case 0xd8e: return "Neoverse-N3";
+    case 0xd8f: return "Cortex-A320";
+    case 0xd90: return "C1-Premium";
     default: return NULL;
     }
 }
@@ -206,8 +217,9 @@ static const char* nvidiaPartId2name(uint32_t partId)
     switch (partId)
     {
     case 0x000: return "Denver";
-    case 0x003: return "Denver 2";
+    case 0x003: return "Denver-2";
     case 0x004: return "Carmel";
+    case 0x010: return "Olympus";
     default: return NULL;
     }
 }
@@ -314,6 +326,7 @@ static const char* fujitsuPartId2name(uint32_t partId)
     switch (partId)
     {
     case 0x001: return "A64FX";
+    case 0x003: return "MONAKA";
     default: return NULL;
     }
 }
