@@ -1,41 +1,21 @@
 #include "terminalshell.h"
-#include "common/io/io.h"
+#include "common/io.h"
 #include "common/processing.h"
 #include "common/thread.h"
-#include "util/mallocHelper.h"
-#include "util/windows/registry.h"
-#include "util/windows/unicode.h"
+#include "common/mallocHelper.h"
+#include "common/windows/registry.h"
+#include "common/windows/unicode.h"
+#include "common/windows/version.h"
+#include "common/windows/nt.h"
+#include "common/stringUtils.h"
 
+#include <stdalign.h>
 #include <windows.h>
 #include <wchar.h>
 #include <tlhelp32.h>
-
-static bool getProductVersion(const wchar_t* filePath, FFstrbuf* version)
-{
-    DWORD handle;
-    DWORD size = GetFileVersionInfoSizeW(filePath, &handle);
-    if(size > 0)
-    {
-        FF_AUTO_FREE void* versionData = malloc(size);
-        if(GetFileVersionInfoW(filePath, handle, size, versionData))
-        {
-            VS_FIXEDFILEINFO* verInfo;
-            UINT len;
-            if(VerQueryValueW(versionData, L"\\", (void**)&verInfo, &len) && len && verInfo->dwSignature == 0xFEEF04BD)
-            {
-                ffStrbufAppendF(version, "%u.%u.%u.%u",
-                    (unsigned)(( verInfo->dwProductVersionMS >> 16 ) & 0xffff),
-                    (unsigned)(( verInfo->dwProductVersionMS >>  0 ) & 0xffff),
-                    (unsigned)(( verInfo->dwProductVersionLS >> 16 ) & 0xffff),
-                    (unsigned)(( verInfo->dwProductVersionLS >>  0 ) & 0xffff)
-                );
-                return true;
-            }
-        }
-    }
-
-    return false;
-}
+#include <ntstatus.h>
+#include <winternl.h>
+#include <shlobj.h>
 
 bool fftsGetShellVersion(FFstrbuf* exe, const char* exeName, FFstrbuf* version);
 
@@ -53,13 +33,11 @@ static uint32_t getShellInfo(FFShellResult* result, uint32_t pid)
         if(
             ffStrbufIgnCaseEqualS(&result->prettyName, "sudo")          ||
             ffStrbufIgnCaseEqualS(&result->prettyName, "su")            ||
-            ffStrbufIgnCaseEqualS(&result->prettyName, "sshd")          ||
             ffStrbufIgnCaseEqualS(&result->prettyName, "gdb")           ||
             ffStrbufIgnCaseEqualS(&result->prettyName, "lldb")          ||
             ffStrbufIgnCaseEqualS(&result->prettyName, "python")        || // python on windows generates shim executables
             ffStrbufIgnCaseEqualS(&result->prettyName, "fastfetch")     || // scoop warps the real binaries with a "shim" exe
             ffStrbufIgnCaseEqualS(&result->prettyName, "flashfetch")    ||
-            ffStrbufIgnCaseEqualS(&result->prettyName, "hyfetch")       || // uses fastfetch as backend
             ffStrbufContainIgnCaseS(&result->prettyName, "debug")       ||
             ffStrbufContainIgnCaseS(&result->prettyName, "time")        ||
             ffStrbufStartsWithIgnCaseS(&result->prettyName, "ConEmu") // https://github.com/fastfetch-cli/fastfetch/issues/488#issuecomment-1619982014
@@ -113,7 +91,7 @@ static void setShellInfoDetails(FFShellResult* result)
                     if(wcsncmp(module.szModule, L"clink_dll_", strlen("clink_dll_")) == 0)
                     {
                         ffStrbufAppendS(&result->prettyName, " (with Clink ");
-                        getProductVersion(module.szExePath, &result->prettyName);
+                        ffGetFileVersion(module.szExePath, NULL, &result->prettyName);
                         ffStrbufAppendC(&result->prettyName, ')');
                         break;
                     }
@@ -155,7 +133,7 @@ static bool getTerminalFromEnv(FFTerminalResult* result)
     }
 
     //SSH
-    if(getenv("SSH_CONNECTION") != NULL)
+    if(getenv("SSH_TTY") != NULL)
         term = getenv("SSH_TTY");
 
     //Windows Terminal
@@ -213,17 +191,29 @@ static bool detectDefaultTerminal(FFTerminalResult* result)
                 {
                     ffStrbufSetS(&result->processName, "WindowsTerminal.exe");
                     ffStrbufSetS(&result->prettyName, "WindowsTerminal");
-                    ffStrbufSetF(&result->exe, "%s\\WindowsApps\\%s\\WindowsTerminal.exe", getenv("ProgramFiles"), path.chars);
-                    if(ffPathExists(result->exe.chars, FF_PATHTYPE_FILE))
+
+                    PWSTR programFiles = NULL;
+                    if (SUCCEEDED(SHGetKnownFolderPath(&FOLDERID_ProgramFiles, KF_FLAG_DEFAULT, NULL, &programFiles)))
                     {
-                        result->exeName = result->exe.chars + ffStrbufLastIndexC(&result->exe, '\\') + 1;
-                        ffStrbufSet(&result->exePath, &result->exe);
-                    }
-                    else
-                    {
-                        ffStrbufDestroy(&result->exe);
-                        ffStrbufInitMove(&result->exe, &path);
-                        result->exeName = "";
+                        ffStrbufSetWS(&result->exe, programFiles);
+                        CoTaskMemFree(programFiles);
+                        programFiles = NULL;
+
+                        ffStrbufAppendS(&result->exe, "\\WindowsApps\\");
+                        ffStrbufAppend(&result->exe, &path);
+                        ffStrbufAppendS(&result->exe, "\\WindowsTerminal.exe");
+
+                        if(ffPathExists(result->exe.chars, FF_PATHTYPE_FILE))
+                        {
+                            result->exeName = result->exe.chars + ffStrbufLastIndexC(&result->exe, '\\') + 1;
+                            ffStrbufSet(&result->exePath, &result->exe);
+                        }
+                        else
+                        {
+                            ffStrbufDestroy(&result->exe);
+                            ffStrbufInitMove(&result->exe, &path);
+                            result->exeName = "";
+                        }
                     }
                     return true;
                 }
@@ -231,14 +221,19 @@ static bool detectDefaultTerminal(FFTerminalResult* result)
         }
     }
 
-conhost:
-    ffStrbufSetF(&result->exe, "%s\\System32\\conhost.exe", getenv("SystemRoot"));
-    if(ffPathExists(result->exe.chars, FF_PATHTYPE_FILE))
+conhost:;
+    ULONG_PTR conhostPid = 0;
+    ULONG size;
+    if(NT_SUCCESS(NtQueryInformationProcess(NtCurrentProcess(), ProcessConsoleHostProcess, &conhostPid, sizeof(conhostPid), &size)) && conhostPid != 0)
     {
-        ffStrbufSetS(&result->processName, "conhost.exe");
-        ffStrbufSetS(&result->prettyName, "conhost");
-        result->exeName = result->exe.chars + ffStrbufLastIndexC(&result->exe, '\\') + 1;
-        return true;
+        // For Windows Terminal, it reports the PID of OpenConsole
+        if(ffProcessGetInfoWindows((uint32_t) conhostPid, NULL, &result->processName, &result->exe, &result->exeName, &result->exePath, NULL))
+        {
+            ffStrbufSet(&result->prettyName, &result->processName);
+            if(ffStrbufEndsWithIgnCaseS(&result->prettyName, ".exe"))
+                ffStrbufSubstrBefore(&result->prettyName, result->prettyName.length - 4);
+            return true;
+        }
     }
 
     ffStrbufClear(&result->exe);
@@ -315,6 +310,11 @@ static void setTerminalInfoDetails(FFTerminalResult* result)
         ffStrbufSetStatic(&result->prettyName, "Windows Explorer");
     else if(ffStrbufEqualS(&result->prettyName, "wezterm-gui"))
         ffStrbufSetStatic(&result->prettyName, "WezTerm");
+    else if(ffStrbufIgnCaseEqualS(&result->prettyName, "sshd") || ffStrbufStartsWithIgnCaseS(&result->prettyName, "sshd-"))
+    {
+        const char* tty = getenv("SSH_TTY");
+        if (tty) ffStrbufSetS(&result->prettyName, tty);
+    }
 }
 
 bool fftsGetTerminalVersion(FFstrbuf* processName, FFstrbuf* exe, FFstrbuf* version);
@@ -341,6 +341,10 @@ const FFShellResult* ffDetectShell(void)
     if(!ffProcessGetInfoWindows(0, &ppid, NULL, NULL, NULL, NULL, NULL))
         return &result;
 
+    const char* ignoreParent = getenv("FFTS_IGNORE_PARENT");
+    if (ignoreParent && ffStrEquals(ignoreParent, "1"))
+        ffProcessGetInfoWindows(ppid, &ppid, NULL, NULL, NULL, NULL, NULL);
+
     ppid = getShellInfo(&result, ppid);
 
     if (result.processName.length > 0)
@@ -350,7 +354,7 @@ const FFShellResult* ffDetectShell(void)
         strcpy(tmp, result.exeName);
         char* ext = strrchr(tmp, '.');
         if (ext) *ext = '\0';
-        fftsGetShellVersion(&result.exe, tmp, &result.version);
+        fftsGetShellVersion(result.exePath.length > 0 ? &result.exePath : &result.exe, tmp, &result.version);
     }
 
     return &result;
@@ -386,7 +390,7 @@ const FFTerminalResult* ffDetectTerminal(void)
     if(result.processName.length > 0)
     {
         setTerminalInfoDetails(&result);
-        fftsGetTerminalVersion(&result.processName, &result.exe, &result.version);
+        fftsGetTerminalVersion(&result.processName, result.exePath.length > 0 ? &result.exePath : &result.exe, &result.version);
     }
 
     return &result;
