@@ -1,8 +1,10 @@
 #include "image.h"
-#include "common/io/io.h"
+#include "common/io.h"
 #include "common/printing.h"
-#include "util/stringUtils.h"
-#include "util/base64.h"
+#include "common/processing.h"
+#include "common/stringUtils.h"
+#include "common/base64.h"
+#include "detection/terminalsize/terminalsize.h"
 
 #include <limits.h>
 #include <math.h>
@@ -77,12 +79,14 @@ static bool printImageIterm(bool printError)
         if (options->position == FF_LOGO_POSITION_LEFT || options->position == FF_LOGO_POSITION_RIGHT)
         {
             uint16_t X = 0, Y = 0;
-            const char* error = ffGetTerminalResponse("\e[6n", "\e[%hu;%huR", &Y, &X);
+            const char* error = ffGetTerminalResponse("\e[6n", 2, "%*[^0-9]%hu;%huR", &Y, &X);
             if (error)
             {
                 fprintf(stderr, "\nLogo (iterm): fail to query cursor position: %s\n", error);
                 return true; // We already printed image logo, don't print ascii logo then
             }
+            if (X < options->paddingLeft + options->width)
+                X = (uint16_t) (options->paddingLeft + options->width);
             if (options->position == FF_LOGO_POSITION_LEFT)
                 instance.state.logoWidth = X + options->paddingRight - 1;
             instance.state.logoHeight = Y;
@@ -135,6 +139,127 @@ static bool printImageIterm(bool printError)
     return true;
 }
 
+static bool printImageKittyIcat(bool printError)
+{
+    const FFOptionsLogo* options = &instance.config.logo;
+
+    if (!ffPathExists(options->source.chars, FF_PATHTYPE_FILE))
+    {
+        if (printError)
+            fputs("Logo (kitty-icat): Failed to load image file\n", stderr);
+        return false;
+    }
+
+    fflush(stdout);
+
+    FF_STRBUF_AUTO_DESTROY buf = ffStrbufCreate();
+
+    if (options->position == FF_LOGO_POSITION_LEFT)
+    {
+        ffStrbufAppendF(&buf, "\e[2J\e[3J\e[%u;%uH",
+            (unsigned) options->paddingTop + 1,
+            (unsigned) options->paddingLeft + 1
+        );
+    }
+    else if (options->position == FF_LOGO_POSITION_TOP)
+    {
+        if (!options->width)
+        {
+            ffStrbufAppendNC(&buf, options->paddingTop, '\n');
+            ffStrbufAppendNC(&buf, options->paddingLeft, ' ');
+        }
+        else
+        {
+            if (printError)
+                fputs("Logo (kitty-icat): position top is not supported when logo width is set\n", stderr);
+            return false;
+        }
+    }
+    else if (options->position == FF_LOGO_POSITION_RIGHT)
+    {
+        if (printError)
+            fputs("Logo (kitty-icat): position right is not supported\n", stderr);
+        return false;
+    }
+
+    uint32_t prevLength = buf.length;
+
+    const char* error = NULL;
+
+    if (options->width)
+    {
+        char place[64];
+        snprintf(place,
+            ARRAY_SIZE(place),
+            "--place=%ux%u@%ux%u",
+            options->width,
+            options->height == 0 ? 9999 : options->height,
+            options->paddingLeft + 1,
+            options->paddingTop + 1);
+
+        error = ffProcessAppendStdOut(&buf, (char* []) {
+            "kitten",
+            "icat",
+            "-n",
+            "--align=center",
+            place,
+            "--scale-up",
+            options->source.chars,
+            NULL,
+        });
+    }
+    else
+    {
+        error = ffProcessAppendStdOut(&buf, (char* []) {
+            "kitten",
+            "icat",
+            "-n",
+            "--align=left",
+            options->source.chars,
+            NULL,
+        });
+    }
+    if (error)
+    {
+        if (printError)
+            fprintf(stderr, "Logo (kitty-icat): running `kitten icat` failed %s\n", error);
+        return false;
+    }
+
+    if (buf.length == prevLength)
+    {
+        if (printError)
+            fputs("Logo (kitty-icat): `kitten icat` returned empty output\n", stderr);
+        return false;
+    }
+
+    ffWriteFDBuffer(FFUnixFD2NativeFD(STDOUT_FILENO), &buf);
+
+    if (options->position == FF_LOGO_POSITION_LEFT || options->position == FF_LOGO_POSITION_RIGHT)
+    {
+        uint16_t X = 0, Y = 0;
+        const char* error = ffGetTerminalResponse("\e[6n", 2, "%*[^0-9]%hu;%huR", &Y, &X);
+        if (error)
+        {
+            fprintf(stderr, "\nLogo (kitty-icat): fail to query cursor position: %s\n", error);
+            return true; // We already printed image logo, don't print ascii logo then
+        }
+        if (X < options->paddingLeft + options->width)
+            X = (uint16_t) (options->paddingLeft + options->width);
+        if (options->position == FF_LOGO_POSITION_LEFT)
+            instance.state.logoWidth = X + options->paddingRight - 1;
+        instance.state.logoHeight = Y;
+        fputs("\e[H", stdout);
+    }
+    else if (options->position == FF_LOGO_POSITION_TOP)
+    {
+        instance.state.logoWidth = instance.state.logoHeight = 0;
+        ffPrintCharTimes('\n', options->paddingRight);
+    }
+
+    return true;
+}
+
 static bool printImageKittyDirect(bool printError)
 {
     const FFOptionsLogo* options = &instance.config.logo;
@@ -200,13 +325,15 @@ static bool printImageKittyDirect(bool printError)
         if (options->position == FF_LOGO_POSITION_LEFT || options->position == FF_LOGO_POSITION_RIGHT)
         {
             uint16_t X = 0, Y = 0;
-            const char* error = ffGetTerminalResponse("\e[6n", "\e[%hu;%huR", &Y, &X);
+            const char* error = ffGetTerminalResponse("\e[6n", 2, "%*[^0-9]%hu;%huR", &Y, &X);
             if (error)
             {
                 if (printError)
                     fprintf(stderr, "\nLogo (kitty-direct): fail to query cursor position: %s\n", error);
                 return true; // We already printed image logo, don't print ascii logo then
             }
+            if (X < options->paddingLeft + options->width)
+                X = (uint16_t) (options->paddingLeft + options->width);
             if (options->position == FF_LOGO_POSITION_LEFT)
                 instance.state.logoWidth = X + options->paddingRight - 1;
             instance.state.logoHeight = Y;
@@ -300,7 +427,7 @@ static inline char* realpath(const char* restrict file_name, char* restrict reso
 
 static bool compressBlob(void** blob, size_t* length)
 {
-    FF_LIBRARY_LOAD(zlib, &instance.config.library.libZ, false, "libz" FF_LIBRARY_EXTENSION, 2)
+    FF_LIBRARY_LOAD(zlib, false, "libz" FF_LIBRARY_EXTENSION, 2)
     FF_LIBRARY_LOAD_SYMBOL(zlib, compressBound, false)
     FF_LIBRARY_LOAD_SYMBOL(zlib, compress2, false)
 
@@ -485,7 +612,7 @@ static bool printImageKitty(FFLogoRequestData* requestData, const ImageData* ima
 #include <chafa.h>
 static bool printImageChafa(FFLogoRequestData* requestData, const ImageData* imageData)
 {
-    FF_LIBRARY_LOAD(chafa, &instance.config.library.libChafa, false,
+    FF_LIBRARY_LOAD(chafa, false,
         "libchafa" FF_LIBRARY_EXTENSION, 1,
         "libchafa-0" FF_LIBRARY_EXTENSION, -1 // Required for Windows
     )
@@ -581,8 +708,8 @@ static bool printImageChafa(FFLogoRequestData* requestData, const ImageData* ima
 
 FFLogoImageResult ffLogoPrintImageImpl(FFLogoRequestData* requestData, const FFIMData* imData)
 {
-    FF_LIBRARY_LOAD_SYMBOL(imData->library, MagickCoreGenesis, FF_LOGO_IMAGE_RESULT_INIT_ERROR);
-    FF_LIBRARY_LOAD_SYMBOL(imData->library, MagickCoreTerminus, FF_LOGO_IMAGE_RESULT_INIT_ERROR);
+    FF_LIBRARY_LOAD_SYMBOL(imData->library, MagickCoreGenesis, FF_LOGO_IMAGE_RESULT_INIT_ERROR)
+    FF_LIBRARY_LOAD_SYMBOL(imData->library, MagickCoreTerminus, FF_LOGO_IMAGE_RESULT_INIT_ERROR)
     FF_LIBRARY_LOAD_SYMBOL(imData->library, AcquireExceptionInfo, FF_LOGO_IMAGE_RESULT_INIT_ERROR)
     FF_LIBRARY_LOAD_SYMBOL(imData->library, DestroyExceptionInfo, FF_LOGO_IMAGE_RESULT_INIT_ERROR)
     FF_LIBRARY_LOAD_SYMBOL(imData->library, AcquireImageInfo, FF_LOGO_IMAGE_RESULT_INIT_ERROR)
@@ -764,13 +891,13 @@ static bool printCachedPixel(FFLogoRequestData* requestData)
     if(requestData->type == FF_LOGO_TYPE_IMAGE_KITTY)
     {
         fd = getCacheFD(requestData, FF_CACHE_FILE_KITTY_COMPRESSED);
-        if(fd == FF_INVALID_FD)
+        if(!ffIsValidNativeFD(fd))
             fd = getCacheFD(requestData, FF_CACHE_FILE_KITTY_UNCOMPRESSED);
     }
     else if(requestData->type == FF_LOGO_TYPE_IMAGE_SIXEL)
         fd = getCacheFD(requestData, FF_CACHE_FILE_SIXEL);
 
-    if(fd == FF_INVALID_FD)
+    if(!ffIsValidNativeFD(fd))
         return false;
 
     ffPrintCharTimes('\n', options->paddingTop);
@@ -831,38 +958,25 @@ static bool printCached(FFLogoRequestData* requestData)
 
 static bool getCharacterPixelDimensions(FFLogoRequestData* requestData)
 {
-    #ifndef _WIN32
-
-    struct winsize winsize;
-
-    //Initialize every member to 0, because it isn't guaranteed that every terminal sets them all
-    memset(&winsize, 0, sizeof(struct winsize));
-
-    ioctl(STDOUT_FILENO, TIOCGWINSZ, &winsize);
-
-    if(winsize.ws_row == 0 || winsize.ws_col == 0)
-        ffGetTerminalResponse("\033[18t", "\033[8;%hu;%hut", &winsize.ws_row, &winsize.ws_col);
-
-    if(winsize.ws_row == 0 || winsize.ws_col == 0)
-        return false;
-
-    if(winsize.ws_ypixel == 0 || winsize.ws_xpixel == 0)
-        ffGetTerminalResponse("\033[14t", "\033[4;%hu;%hut", &winsize.ws_ypixel, &winsize.ws_xpixel);
-
-    requestData->characterPixelWidth = winsize.ws_xpixel / (double) winsize.ws_col;
-    requestData->characterPixelHeight = winsize.ws_ypixel / (double) winsize.ws_row;
-
-    #else
+    #ifdef _WIN32
 
     CONSOLE_FONT_INFO cfi;
-    if(GetCurrentConsoleFont(GetStdHandle(STD_OUTPUT_HANDLE), FALSE, &cfi) == FALSE) // Only works for ConHost
-        return false;
-
-    requestData->characterPixelWidth = cfi.dwFontSize.X;
-    requestData->characterPixelHeight = cfi.dwFontSize.Y;
-
+    if(GetCurrentConsoleFont(GetStdHandle(STD_OUTPUT_HANDLE), FALSE, &cfi)) // Only works for ConHost
+    {
+        requestData->characterPixelWidth = cfi.dwFontSize.X;
+        requestData->characterPixelHeight = cfi.dwFontSize.Y;
+    }
+    if (requestData->characterPixelWidth > 1.0 && requestData->characterPixelHeight > 1.0)
+        return true;
     #endif
 
+    FFTerminalSizeResult termSize = {};
+    if (ffDetectTerminalSize(&termSize))
+    {
+        requestData->characterPixelWidth = termSize.width / (double) termSize.columns;
+        requestData->characterPixelHeight = termSize.height / (double) termSize.rows;
+    }
+
     return requestData->characterPixelWidth > 1.0 && requestData->characterPixelHeight > 1.0;
 }
 
@@ -965,6 +1079,9 @@ bool ffLogoPrintImageIfExists(FFLogoType type, bool printError)
     if(type == FF_LOGO_TYPE_IMAGE_KITTY_DIRECT)
         return printImageKittyDirect(printError);
 
+    if(type == FF_LOGO_TYPE_IMAGE_KITTY_ICAT)
+        return printImageKittyIcat(printError);
+
     #if !defined(FF_HAVE_CHAFA)
         if(type == FF_LOGO_TYPE_IMAGE_CHAFA)
         {
