@@ -2,9 +2,9 @@
 #include "common/jsonconfig.h"
 #include "common/percent.h"
 #include "common/size.h"
+#include "common/stringUtils.h"
 #include "detection/memory/memory.h"
 #include "modules/memory/memory.h"
-#include "util/stringUtils.h"
 
 bool ffPrintMemory(FFMemoryOptions* options)
 {
