@@ -4,7 +4,7 @@
 
 #define FF_OS_MODULE_NAME "OS"
 
-void ffPrintOS(FFOSOptions* options);
+bool ffPrintOS(FFOSOptions* options);
 void ffInitOSOptions(FFOSOptions* options);
 void ffDestroyOSOptions(FFOSOptions* options);
 
